@@ -28,6 +28,7 @@ import logging
 
 from autopilot.display import is_point_on_any_screen, move_mouse_to_screen
 from autopilot.utilities import (
+    EventDelay,
     Silence,
     sleep,
     StagnantStateDetector,
@@ -286,6 +287,7 @@ class Mouse(MouseBase):
         super(Mouse, self).__init__()
         # Try to access the screen to see if X11 mouse is supported
         get_display()
+        self.event_delayer = EventDelay()
 
     @property
     def x(self):
@@ -316,8 +318,9 @@ class Mouse(MouseBase):
         fake_input(get_display(), X.ButtonRelease, button)
         get_display().sync()
 
-    def click(self, button=1, press_duration=0.10):
+    def click(self, button=1, press_duration=0.10, time_between_events=0.1):
         """Click mouse at current location."""
+        self.event_delayer.delay(time_between_events)
         self.press(button)
         sleep(press_duration)
         self.release(button)
@@ -386,6 +389,9 @@ class Mouse(MouseBase):
                 e.args = ("Mouse cursor is stuck.", )
                 raise
 
+        x, y = self.position()
+        _logger.debug('The mouse is now at position %d,%d.', x, y)
+
     def move_to_object(self, object_proxy):
         """Attempts to move the mouse to 'object_proxy's centre point.
 
