@@ -1,7 +1,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Autopilot Functional Test Tool
-# Copyright (C) 2012-2013 Canonical
+# Copyright (C) 2012, 2013, 2014, 2015 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ from autopilot.input._common import get_center_point
 
 import logging
 
-logger = logging.getLogger(__name__)
+_logger = logging.getLogger(__name__)
 
 
 class Keyboard(CleanupRegistered):
@@ -129,7 +129,7 @@ class Keyboard(CleanupRegistered):
         return _pick_backend(backends, preferred_backend)
 
     @contextmanager
-    def focused_type(self, input_target,  pointer=None):
+    def focused_type(self, input_target, pointer=None):
         """Type into an input widget.
 
         This context manager takes care of making sure a particular
@@ -176,9 +176,9 @@ class Keyboard(CleanupRegistered):
         .. warning:: The **OSK** backend does not implement the press method
           and will raise a NotImplementedError if called.
 
-        Example:
+        Example::
 
-        >>> press('Alt+F2')
+            press('Alt+F2')
 
         presses the 'Alt' and 'F2' keys, but does not release them.
 
@@ -196,9 +196,9 @@ class Keyboard(CleanupRegistered):
         .. warning:: The **OSK** backend does not implement the press method
          and will raise a NotImplementedError if called.
 
-        Example:
+        Example::
 
-        >>> release('Alt+F2')
+            release('Alt+F2')
 
         releases the 'Alt' and 'F2' keys.
 
@@ -214,9 +214,9 @@ class Keyboard(CleanupRegistered):
         :param delay: The delay (in Seconds) after pressing and releasing each
             key.
 
-        Example:
+        Example::
 
-        >>> press_and_release('Alt+F2')
+            press_and_release('Alt+F2')
 
         presses both the 'Alt' and 'F2' keys, and then releases both keys.
 
@@ -248,11 +248,11 @@ class Mouse(CleanupRegistered):
     test. This class should not be instantiated directly however. To get an
     instance of the mouse class, call :py:meth:`create` instead.
 
-    For example, to create a mouse object and click at (100,50):
+    For example, to create a mouse object and click at (100,50)::
 
-    >>> mouse = autopilot.input.Mouse.create()
-    >>> mouse.move(100, 50)
-    >>> mouse.click()
+        mouse = Mouse.create()
+        mouse.move(100, 50)
+        mouse.click()
 
     """
 
@@ -282,7 +282,7 @@ class Mouse(CleanupRegistered):
 
         from autopilot.platform import model
         if model() != 'Desktop':
-            logger.info(
+            _logger.info(
                 "You cannot create a Mouse on the devices where X11 is not "
                 "available. consider using a Touch or Pointer device. "
                 "For more information, see: "
@@ -315,11 +315,22 @@ class Mouse(CleanupRegistered):
         """Releases mouse button at current mouse location."""
         raise NotImplementedError("You cannot use this class directly.")
 
-    def click(self, button=1, press_duration=0.10):
-        """Click mouse at current location."""
+    def click(self, button=1, press_duration=0.10, time_between_events=0.1):
+        """Click mouse at current location.
+
+        :param time_between_events: takes floating point to represent the
+          delay time between subsequent clicks. Default value 0.1 represents
+          tenth of a second.
+
+        """
         raise NotImplementedError("You cannot use this class directly.")
 
-    def click_object(self, object_proxy, button=1, press_duration=0.10):
+    def click_object(
+            self,
+            object_proxy,
+            button=1,
+            press_duration=0.10,
+            time_between_events=0.1):
         """Click the center point of a given object.
 
         It does this by looking for several attributes, in order. The first
@@ -329,12 +340,15 @@ class Mouse(CleanupRegistered):
          * center_x, center_y
          * x, y, w, h
 
+        :param time_between_events: takes floating point to represent the
+          delay time between subsequent clicks. Default value 0.1 represents
+          tenth of a second.
         :raises: **ValueError** if none of these attributes are found, or if an
          attribute is of an incorrect type.
 
          """
         self.move_to_object(object_proxy)
-        self.click(button, press_duration)
+        self.click(button, press_duration, time_between_events)
 
     def move(self, x, y, animate=True, rate=10, time_between_events=0.01):
         """Moves mouse to location (x,y).
@@ -369,12 +383,27 @@ class Mouse(CleanupRegistered):
         """
         raise NotImplementedError("You cannot use this class directly.")
 
-    def drag(self, x1, y1, x2, y2):
-        """Performs a press, move and release.
+    def drag(self, x1, y1, x2, y2, rate=10, time_between_events=0.01):
+        """Perform a press, move and release.
 
         This is to keep a common API between Mouse and Finger as long as
         possible.
 
+        The pointer will be dragged from the starting point to the ending point
+        with multiple moves. The number of moves, and thus the time that it
+        will take to complete the drag can be altered with the `rate`
+        parameter.
+
+        :param x1: The point on the x axis where the drag will start from.
+        :param y1: The point on the y axis where the drag will starts from.
+        :param x2: The point on the x axis where the drag will end at.
+        :param y2: The point on the y axis where the drag will end at.
+        :param rate: The number of pixels the mouse will be moved per
+            iteration. Default is 10 pixels. A higher rate will make the drag
+            faster, and lower rate will make it slower.
+        :param time_between_events: The number of seconds that the drag will
+            wait between iterations.
+
         """
         raise NotImplementedError("You cannot use this class directly.")
 
@@ -425,11 +454,17 @@ class Touch(object):
         """
         raise NotImplementedError("You cannot use this class directly.")
 
-    def tap(self, x, y):
-        """Click (or 'tap') at given x,y coordinates."""
+    def tap(self, x, y, press_duration=0.1, time_between_events=0.1):
+        """Click (or 'tap') at given x,y coordinates.
+
+        :param time_between_events: takes floating point to represent the
+          delay time between subsequent taps. Default value 0.1 represents
+          tenth of a second.
+
+        """
         raise NotImplementedError("You cannot use this class directly.")
 
-    def tap_object(self, object):
+    def tap_object(self, object, press_duration=0.1, time_between_events=0.1):
         """Tap the center point of a given object.
 
         It does this by looking for several attributes, in order. The first
@@ -439,6 +474,9 @@ class Touch(object):
          * center_x, center_y
          * x, y, w, h
 
+        :param time_between_events: takes floating point to represent the
+          delay time between subsequent taps. Default value 0.1 represents
+          tenth of a second.
         :raises: **ValueError** if none of these attributes are found, or if an
          attribute is of an incorrect type.
 
@@ -466,8 +504,28 @@ class Touch(object):
         """Release a previously pressed finger"""
         raise NotImplementedError("You cannot use this class directly.")
 
-    def drag(self, x1, y1, x2, y2):
-        """Perform a drag gesture from (x1,y1) to (x2,y2)"""
+    def drag(self, x1, y1, x2, y2, rate=10, time_between_events=0.01):
+        """Perform a drag gesture.
+
+        The finger will be dragged from the starting point to the ending point
+        with multiple moves. The number of moves, and thus the time that it
+        will take to complete the drag can be altered with the `rate`
+        parameter.
+
+        :param x1: The point on the x axis where the drag will start from.
+        :param y1: The point on the y axis where the drag will starts from.
+        :param x2: The point on the x axis where the drag will end at.
+        :param y2: The point on the y axis where the drag will end at.
+        :param rate: The number of pixels the finger will be moved per
+            iteration. Default is 10 pixels. A higher rate will make the drag
+            faster, and lower rate will make it slower.
+        :param time_between_events: The number of seconds that the drag will
+            wait between iterations.
+
+        :raises RuntimeError: if the finger is already pressed.
+        :raises RuntimeError: if no more finger slots are available.
+
+        """
         raise NotImplementedError("You cannot use this class directly.")
 
 
@@ -500,8 +558,6 @@ class Pointer(object):
             raise TypeError(
                 "`device` must be either a Touch or a Mouse instance.")
         self._device = device
-        self._x = 0
-        self._y = 0
 
     @property
     def x(self):
@@ -511,10 +567,7 @@ class Pointer(object):
         last known X coordinate, which may not be a sensible value.
 
         """
-        if isinstance(self._device, Mouse):
-            return self._device.x
-        else:
-            return self._x
+        return self._device.x
 
     @property
     def y(self):
@@ -524,10 +577,7 @@ class Pointer(object):
         last known Y coordinate, which may not be a sensible value.
 
         """
-        if isinstance(self._device, Mouse):
-            return self._device.y
-        else:
-            return self._y
+        return self._device.y
 
     def press(self, button=1):
         """Press the pointer at it's current location.
@@ -543,7 +593,7 @@ class Pointer(object):
             if button != 1:
                 raise ValueError(
                     "Touch devices do not have button %d" % (button))
-            self._device.press(self._x, self._y)
+            self._device.press(self.x, self.y)
 
     def release(self, button=1):
         """Releases the pointer at it's current location.
@@ -561,21 +611,29 @@ class Pointer(object):
                     "Touch devices do not have button %d" % (button))
             self._device.release()
 
-    def click(self, button=1, press_duration=0.10):
+    def click(self, button=1, press_duration=0.10, time_between_events=0.1):
         """Press and release at the current pointer location.
 
         If the wrapped device is a mouse, the button specification is used. If
         it is a touch device, passing anything other than 1 will raise a
         ValueError exception.
 
+        :param time_between_events: takes floating point to represent the
+          delay time between subsequent clicks/taps. Default value 0.1
+          represents tenth of a second.
         """
         if isinstance(self._device, Mouse):
-            self._device.click(button, press_duration)
+            self._device.click(button, press_duration, time_between_events)
         else:
             if button != 1:
                 raise ValueError(
                     "Touch devices do not have button %d" % (button))
-            self._device.tap(self._x, self._y)
+            self._device.tap(
+                self.x,
+                self.y,
+                press_duration=press_duration,
+                time_between_events=time_between_events
+            )
 
     def move(self, x, y):
         """Moves the pointer to the specified coordinates.
@@ -586,13 +644,14 @@ class Pointer(object):
         :meth:`click` will occur.
 
         """
-        if isinstance(self._device, Mouse):
-            self._device.move(x, y)
-        else:
-            self._x = x
-            self._y = y
+        self._device.move(x, y)
 
-    def click_object(self, object_proxy, button=1, press_duration=0.10):
+    def click_object(
+            self,
+            object_proxy,
+            button=1,
+            press_duration=0.10,
+            time_between_events=0.1):
         """
         Attempts to move the pointer to 'object_proxy's centre point.
         and click a button
@@ -608,10 +667,14 @@ class Pointer(object):
         it is a touch device, passing anything other than 1 will raise a
         ValueError exception.
 
+        :param time_between_events: takes floating point to represent the
+          delay time between subsequent clicks/taps. Default value 0.1
+          represents tenth of a second.
+
         """
 
         self.move_to_object(object_proxy)
-        self.click(button, press_duration)
+        self.click(button, press_duration, time_between_events)
 
     def move_to_object(self, object_proxy):
         """Attempts to move the pointer to 'object_proxy's centre point.
@@ -639,11 +702,29 @@ class Pointer(object):
         if isinstance(self._device, Mouse):
             return self._device.position()
         else:
-            return (self._x, self._y)
+            return (self.x, self.y)
 
-    def drag(self, x1, y1, x2, y2):
-        """Performs a press, move and release."""
-        self._device.drag(x1, y1, x2, y2)
-        if isinstance(self._device, Touch):
-            self._x = x2
-            self._y = y2
+    def drag(self, x1, y1, x2, y2, rate=10, time_between_events=0.01):
+        """Perform a press, move and release.
+
+        This is to keep a common API between Mouse and Finger as long as
+        possible.
+
+        The pointer will be dragged from the starting point to the ending point
+        with multiple moves. The number of moves, and thus the time that it
+        will take to complete the drag can be altered with the `rate`
+        parameter.
+
+        :param x1: The point on the x axis where the drag will start from.
+        :param y1: The point on the y axis where the drag will starts from.
+        :param x2: The point on the x axis where the drag will end at.
+        :param y2: The point on the y axis where the drag will end at.
+        :param rate: The number of pixels the mouse will be moved per
+            iteration. Default is 10 pixels. A higher rate will make the drag
+            faster, and lower rate will make it slower.
+        :param time_between_events: The number of seconds that the drag will
+            wait between iterations.
+
+        """
+        self._device.drag(
+            x1, y1, x2, y2, rate=rate, time_between_events=time_between_events)
