@@ -109,8 +109,8 @@ def _get_parser():
     parser_run.add_argument(
         '-v', '--verbose', default=False, required=False, action='count',
         help="If set, autopilot will output test log data to stderr during a "
-        "test run. Set twice to also log data useful for debugging autopilot "
-        "itself.")
+        "test run. Set twice (i.e. -vv) to also log debug level messages. "
+        "(This can be useful for debugging autopilot itself.)")
     parser_run.add_argument(
         "--debug-profile",
         choices=[p.name for p in get_all_debug_profiles()],
@@ -132,6 +132,13 @@ def _get_parser():
         "of values, where each value is either of the form 'key', or "
         "'key=value'.", dest="test_config"
     )
+    parser_run.add_argument(
+        "--test-timeout", default=0, type=int, help="If set, autopilot will "
+        "attempt to abort tests that have run longer than <test-timeout> "
+        "seconds. This is not guaranteed to succeed - several scenarios exist "
+        "which make it impossible to abort a test case. Tests aborted will "
+        "raise a 'TimeoutException' error."
+    )
     parser_run.add_argument("suite", nargs="+",
                             help="Specify test suite(s) to run.")
 
@@ -217,14 +224,19 @@ class _OneOrMoreArgumentStoreAction(Action):
 def setup_logging(verbose):
     """Configure the root logger and verbose logging to stderr."""
     root_logger = get_root_logger()
-    root_logger.setLevel(logging.DEBUG)
+    root_logger.setLevel(logging.INFO)
     if verbose == 0:
         set_null_log_handler(root_logger)
     if verbose >= 1:
+        autopilot.globals.set_log_verbose(True)
         set_stderr_stream_handler(root_logger)
     if verbose >= 2:
+        root_logger.setLevel(logging.DEBUG)
         enable_debug_log_messages()
-    # log autopilot version
+
+
+def log_autopilot_version():
+    root_logger = get_root_logger()
     root_logger.info(get_version_string())
 
 
@@ -484,6 +496,10 @@ def _configure_timeout_profile(args):
         autopilot.globals.set_long_timeout_period(30.0)
 
 
+def _configure_test_timeout(args):
+    autopilot.globals.set_test_timeout(args.test_timeout)
+
+
 def _prepare_application_for_launch(application, interface):
     app_path, app_arguments = _get_application_path_and_arguments(application)
     return _prepare_launcher_environment(
@@ -600,6 +616,8 @@ class TestProgram(object):
     def run(self):
         setup_logging(getattr(self.args, 'verbose', False))
 
+        log_autopilot_version()
+
         action = None
         if self.args.mode == 'list':
             action = self.list_tests
@@ -651,6 +669,19 @@ class TestProgram(object):
 
     def run_tests(self):
         """Run tests, using input from `args`."""
+
+        _configure_debug_profile(self.args)
+        _configure_timeout_profile(self.args)
+        _configure_test_timeout(self.args)
+
+        try:
+            _video.configure_video_recording(self.args)
+        except RuntimeError as e:
+            print("Error: %s" % str(e))
+            exit(1)
+
+        test_config.set_configuration_string(self.args.test_config)
+
         test_suite, error_encountered = load_test_suite_from_name(
             self.args.suite
         )
@@ -662,19 +693,6 @@ class TestProgram(object):
             shuffle(test_suite._tests)
             print("Running tests in random order")
 
-        _configure_debug_profile(self.args)
-        _configure_timeout_profile(self.args)
-        try:
-            _video.configure_video_recording(self.args)
-        except RuntimeError as e:
-            print("Error: %s" % str(e))
-            exit(1)
-
-        test_config.set_configuration_string(self.args.test_config)
-
-        if self.args.verbose:
-            autopilot.globals.set_log_verbose(True)
-
         result = construct_test_result(self.args)
         result.startTestRun()
         try:
