@@ -23,8 +23,6 @@ import os
 import subprocess
 import logging
 import sys
-import six
-from mock import patch
 from testtools import skipIf
 from testtools.matchers import (
     Equals,
@@ -35,6 +33,13 @@ from testtools.matchers import (
 )
 from textwrap import dedent
 
+from fixtures import EnvironmentVariable
+
+from autopilot.application import (
+    NormalApplicationLauncher,
+    UpstartApplicationLauncher,
+)
+from autopilot.exceptions import ProcessSearchError
 from autopilot.process import ProcessManager
 from autopilot.platform import model
 from autopilot.testcase import AutopilotTestCase
@@ -42,18 +47,10 @@ from autopilot.tests.functional.fixtures import (
     ExecutableScript,
     TempDesktopFile,
 )
-from autopilot.introspection import (
-    get_proxy_object_for_existing_process,
-    ProcessSearchError,
-    _pid_is_running,
-)
+from autopilot.introspection import get_proxy_object_for_existing_process
+from autopilot.introspection.utilities import _pid_is_running
 from autopilot.utilities import sleep
 
-
-# backwards compatible alias for Python 3
-if six.PY3:
-    xrange = range
-
 logger = logging.getLogger(__name__)
 
 
@@ -61,7 +58,7 @@ def locale_is_supported():
     """Check if our currently set locale supports writing unicode to stdout."""
     try:
         encoding = sys.stdout.encoding or sys.getfilesystemencoding()
-        u'\u2026'.encode(encoding)
+        '\u2026'.encode(encoding)
         return True
     except UnicodeEncodeError:
         return False
@@ -73,7 +70,7 @@ def _get_unused_pid():
     :raises: **RuntimeError** if unable to produce a number that doesn't
      correspond to a currently running process.
     """
-    for i in xrange(10000, 20000):
+    for i in range(10000, 20000):
         if not _pid_is_running(i):
             return i
     raise RuntimeError("Unable to find test PID.")
@@ -161,10 +158,6 @@ class ApplicationLaunchTests(ApplicationTests):
             raises(ProcessSearchError(expected_error))
         )
 
-    @patch(
-        'autopilot.introspection._search_for_valid_connections',
-        new=lambda *args: []
-    )
     def test_creating_proxy_for_segfaulted_app_fails_quicker(self):
         """Searching for a process that has died since launching, the search
         must fail before the 10 second timeout.
@@ -194,9 +187,9 @@ class ApplicationLaunchTests(ApplicationTests):
         self.assertThat(difference.total_seconds(), LessThan(5))
 
     @skipIf(model() != "Desktop", "Not suitable for device (Qt4)")
-    def test_closing_app_produces_good_error_from_get_state_by_path(self):
+    def test_closing_app_produces_good_error(self):
         """Testing an application that closes before the test ends must
-        produce a good error message when calling get_state_by_path on the
+        produce a good error message when calling refresh_state() on the
         application proxy object.
 
         """
@@ -218,7 +211,7 @@ class ApplicationLaunchTests(ApplicationTests):
 
         def crashing_fn():
             for i in range(10):
-                logger.debug("%d %r", i, app_proxy.get_state_by_path("/"))
+                logger.debug("%d %r", i, app_proxy.refresh_state())
                 sleep(1)
 
         self.assertThat(
@@ -252,14 +245,14 @@ class QmlTestMixin(object):
             if path:
                 not_found = False
                 self.qml_viewer_app_path = path
-                self.patch_environment("QT_SELECT", "qt4")
+                self.useFixture(EnvironmentVariable("QT_SELECT", "qt4"))
 
         if 'qt5' in qtversions:
             path = check_func('qt5', 'qmlscene')
             if path:
                 not_found = False
                 self.qml_viewer_app_path = path
-                self.patch_environment("QT_SELECT", "qt5")
+                self.useFixture(EnvironmentVariable("QT_SELECT", "qt5"))
 
         if not_found:
             self.skip("Neither qmlviewer nor qmlscene is installed")
@@ -295,9 +288,10 @@ class QtTests(ApplicationTests, QmlTestMixin):
     def test_can_launch_normal_app(self):
         path = self.get_qml_viewer_app_path()
         fixture = self.useFixture(TempDesktopFile(exec_=path,))
-        app_proxy = self.launch_test_application(
+        launcher = self.useFixture(NormalApplicationLauncher())
+        app_proxy = launcher.launch(
             path,
-            '--desktop_file_hint=%s' % fixture.get_desktop_file_path(),
+            ['--desktop_file_hint=%s' % fixture.get_desktop_file_path()],
             app_type='qt'
         )
         self.assertTrue(app_proxy is not None)
@@ -305,7 +299,8 @@ class QtTests(ApplicationTests, QmlTestMixin):
     def test_can_launch_upstart_app(self):
         path = self.get_qml_viewer_app_path()
         fixture = self.useFixture(TempDesktopFile(exec_=path,))
-        self.launch_upstart_application(fixture.get_desktop_file_id())
+        launcher = self.useFixture(UpstartApplicationLauncher())
+        launcher.launch(fixture.get_desktop_file_id())
 
     @skipIf(model() != "Desktop", "Only suitable on Desktop (Qt4)")
     def test_can_launch_normal_qt_script(self):
@@ -378,7 +373,10 @@ class QtTests(ApplicationTests, QmlTestMixin):
         app_proxy = self.launch_test_application(wrapper_path, app_type='qt')
         self.assertTrue(app_proxy is not None)
 
-    @skipIf(not locale_is_supported(), "Current locale is not supported")
+    @skipIf(
+        model() != "Desktop" or not locale_is_supported(),
+        "Current locale is not supported or not on desktop (Qt4)"
+    )
     def test_can_handle_non_unicode_stdout_and_stderr(self):
         path = self.write_script(dedent("""\
             #!%s
