@@ -1,7 +1,8 @@
+
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Autopilot Functional Test Tool
-# Copyright (C) 2012, 2013, 2014 Canonical
+# Copyright (C) 2012, 2013, 2014, 2015 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,17 +20,28 @@
 
 
 import json
+import logging
 import os
 from tempfile import mktemp
 from testtools import TestCase, skipIf
-from testtools.matchers import IsInstance, Equals, raises
+from testtools.matchers import (
+    IsInstance,
+    Equals,
+    raises,
+    GreaterThan
+)
+from testscenarios import TestWithScenarios
 from textwrap import dedent
 from time import sleep
+import timeit
 from unittest import SkipTest
 from unittest.mock import patch
 
+from autopilot import (
+    platform,
+    tests
+)
 from autopilot.display import Display
-from autopilot import platform
 from autopilot.gestures import pinch
 from autopilot.input import Keyboard, Mouse, Pointer, Touch
 from autopilot.input._common import get_center_point
@@ -39,6 +51,22 @@ from autopilot.tests.functional import QmlScriptRunnerMixin
 from autopilot.utilities import on_test_started
 
 
+class ElapsedTimeCounter(object):
+
+    """A simple utility to count the amount of real time that passes."""
+
+    def __enter__(self):
+        self._start_time = timeit.default_timer()
+        return self
+
+    def __exit__(self, *args):
+        pass
+
+    @property
+    def elapsed_time(self):
+        return timeit.default_timer() - self._start_time
+
+
 class InputStackKeyboardBase(AutopilotTestCase):
 
     scenarios = [
@@ -162,9 +190,11 @@ class InputStackKeyboardTypingTests(InputStackKeyboardBase):
             from autopilot.input import _uinput
             return _uinput.Keyboard._device._pressed_keys_ecodes
         else:
-            self.fail("Don't know how to get pressed keys list for backend "
-                      + self.backend
-                      )
+            self.fail(
+                "Don't know how to get pressed keys list for {}".format(
+                    self.backend
+                )
+            )
 
 
 @skipIf(platform.model() != "Desktop", "Only suitable on Desktop (WinMocker)")
@@ -324,7 +354,22 @@ class OSKBackendTests(AutopilotTestCase, QmlScriptRunnerMixin):
         )
 
 
-class MouseTestCase(AutopilotTestCase):
+class MockAppMouseTestBase(AutopilotTestCase):
+
+    def start_mock_app(self):
+        window_spec_file = mktemp(suffix='.json')
+        window_spec = {"Contents": "MouseTest"}
+        json.dump(
+            window_spec,
+            open(window_spec_file, 'w')
+        )
+        self.addCleanup(os.remove, window_spec_file)
+
+        return self.launch_test_application(
+            'window-mocker', window_spec_file, app_type='qt')
+
+
+class MouseTestCase(AutopilotTestCase, tests.LogHandlerTestCase):
 
     @skipIf(platform.model() != "Desktop", "Only suitable on Desktop (Mouse)")
     def test_move_to_nonint_point(self):
@@ -333,8 +378,8 @@ class MouseTestCase(AutopilotTestCase):
         LP bug #1195499.
 
         """
-        screen_geometry = Display.create().get_screen_geometry(0)
         device = Mouse.create()
+        screen_geometry = Display.create().get_screen_geometry(0)
         target_x = screen_geometry[0] + 10
         target_y = screen_geometry[1] + 10.6
         device.move(target_x, target_y)
@@ -352,9 +397,19 @@ class MouseTestCase(AutopilotTestCase):
         self.assertThat(lambda: Mouse.create(),
                         raises(expected_exception))
 
+    @skipIf(platform.model() != "Desktop", "Only suitable on Desktop (Mouse)")
+    def test_mouse_move_must_log_final_position_at_debug_level(self):
+        self.root_logger.setLevel(logging.DEBUG)
+        mouse = Mouse.create()
+        mouse.move(10, 10)
+        self.assertLogLevelContains(
+            'DEBUG',
+            "The mouse is now at position 10,10."
+        )
+
 
 @skipIf(platform.model() != "Desktop", "Only suitable on Desktop (WinMocker)")
-class TouchTests(AutopilotTestCase):
+class TouchTests(MockAppMouseTestBase):
 
     def setUp(self):
         super(TouchTests, self).setUp()
@@ -365,18 +420,6 @@ class TouchTests(AutopilotTestCase):
         self.button_status = self.app.select_single(
             'QLabel', objectName='button_status')
 
-    def start_mock_app(self):
-        window_spec_file = mktemp(suffix='.json')
-        window_spec = {"Contents": "MouseTest"}
-        json.dump(
-            window_spec,
-            open(window_spec_file, 'w')
-        )
-        self.addCleanup(os.remove, window_spec_file)
-
-        return self.launch_test_application(
-            'window-mocker', window_spec_file, app_type='qt')
-
     def test_tap(self):
         x, y = get_center_point(self.widget)
         self.device.tap(x, y)
@@ -441,8 +484,8 @@ class PointerWrapperTests(AutopilotTestCase):
         device = Pointer(Touch.create())
         device.move(34, 56)
 
-        self.assertThat(device._x, Equals(34))
-        self.assertThat(device._y, Equals(56))
+        self.assertThat(device.x, Equals(34))
+        self.assertThat(device.y, Equals(56))
 
     def test_touch_drag_updates_coordinates(self):
         """The Pointer wrapper must update it's x and y properties when
@@ -451,11 +494,13 @@ class PointerWrapperTests(AutopilotTestCase):
         """
         class FakeTouch(Touch):
             def __init__(self):
-                pass
+                self.x = 0
+                self.y = 0
 
             def drag(self, x1, y1, x2, y2, rate='dummy',
                      time_between_events='dummy'):
-                pass
+                self.x = x2
+                self.y = y2
 
         p = Pointer(FakeTouch())
         p.drag(0, 0, 100, 123)
@@ -463,6 +508,38 @@ class PointerWrapperTests(AutopilotTestCase):
         self.assertThat(p.y, Equals(123))
 
 
+@skipIf(platform.model() != "Desktop", "Window mocker only available on X11")
+class InputEventDelayTests(MockAppMouseTestBase, TestWithScenarios):
+
+    scenarios = [
+        ('Touch', dict(input_class=Touch)),
+        ('Mouse', dict(input_class=Mouse)),
+    ]
+
+    def setUp(self):
+        super(InputEventDelayTests, self).setUp()
+        self.device = Pointer(self.input_class.create())
+        self.widget = self.get_mock_app_main_widget()
+
+    def get_mock_app_main_widget(self):
+        self.app = self.start_mock_app()
+        return self.app.select_single('MouseTestWidget')
+
+    def test_subsequent_events_delay(self):
+        with ElapsedTimeCounter() as time_counter:
+            for i in range(3):
+                self.device.click_object(self.widget, time_between_events=0.6)
+
+        self.assertThat(time_counter.elapsed_time, GreaterThan(1.0))
+
+    def test_subsequent_events_default_delay(self):
+        with ElapsedTimeCounter() as time_counter:
+            for i in range(10):
+                self.device.click_object(self.widget)
+
+        self.assertThat(time_counter.elapsed_time, GreaterThan(0.9))
+
+
 class InputStackCleanupTests(TestCase):
 
     def test_cleanup_called(self):
