@@ -1,7 +1,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Autopilot Functional Test Tool
-# Copyright (C) 2013, 2014 Canonical
+# Copyright (C) 2013, 2014, 2015 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+import logging
+
 import testscenarios
 from evdev import ecodes, uinput
 from unittest.mock import ANY, call, Mock, patch
@@ -24,7 +26,10 @@ from testtools import TestCase
 from testtools.matchers import Contains, raises
 
 import autopilot.input
-from autopilot import utilities
+from autopilot import (
+    tests,
+    utilities
+)
 from autopilot.input import _uinput
 from autopilot.input._common import get_center_point
 
@@ -420,7 +425,7 @@ class TouchEventsTestCase(TestCase):
         self.assert_expected_ev_abs('given_res_x', 'given_res_y', ev_abs)
 
 
-class UInputTouchDeviceTestCase(TestCase):
+class UInputTouchDeviceTestCase(tests.LogHandlerTestCase):
     """Test the integration with evdev.UInput for the touch device."""
 
     def setUp(self):
@@ -463,7 +468,7 @@ class UInputTouchDeviceTestCase(TestCase):
             call.write(
                 ecodes.EV_ABS, ecodes.ABS_MT_TRACKING_ID, tracking_id),
             call.write(
-                ecodes.EV_KEY, ecodes.BTN_TOOL_FINGER, press_value),
+                ecodes.EV_KEY, ecodes.BTN_TOUCH, press_value),
             call.write(ecodes.EV_ABS, ecodes.ABS_MT_POSITION_X, x),
             call.write(ecodes.EV_ABS, ecodes.ABS_MT_POSITION_Y, y),
             call.write(ecodes.EV_ABS, ecodes.ABS_MT_PRESSURE, 400),
@@ -488,7 +493,7 @@ class UInputTouchDeviceTestCase(TestCase):
             call.write(
                 ecodes.EV_ABS, ecodes.ABS_MT_TRACKING_ID, lift_tracking_id),
             call.write(
-                ecodes.EV_KEY, ecodes.BTN_TOOL_FINGER, release_value),
+                ecodes.EV_KEY, ecodes.BTN_TOUCH, release_value),
             call.syn()
         ]
         self.assertEqual(expected_calls, touch._device.mock_calls)
@@ -577,6 +582,21 @@ class UInputTouchDeviceTestCase(TestCase):
         self.assert_finger_move_emitted_write_and_syn(
             touch, slot=first_slot, x=14, y=14)
 
+    def test_finger_move_must_log_position_at_debug_level(self):
+        self.root_logger.setLevel(logging.DEBUG)
+        touch = self.get_touch_with_mocked_backend()
+        touch.finger_down(0, 0)
+
+        touch.finger_move(10, 10)
+        self.assertLogLevelContains(
+            'DEBUG',
+            "Moving pointing 'finger' to position 10,10."
+        )
+        self.assertLogLevelContains(
+            'DEBUG',
+            "The pointing 'finger' is now at position 10,10."
+        )
+
     def test_finger_up_without_finger_pressed_must_raise_error(self):
         touch = self.get_touch_with_mocked_backend()
 
@@ -641,35 +661,83 @@ class UInputTouchDeviceTestCase(TestCase):
         self.assertFalse(touch.pressed)
 
 
-class UInputTouchTestCase(TestCase):
-    """Test UInput Touch helper for autopilot tests."""
+class UInputTouchBaseTestCase(TestCase):
 
     def setUp(self):
-        super(UInputTouchTestCase, self).setUp()
+        super(UInputTouchBaseTestCase, self).setUp()
         # Mock the sleeps so we don't have to spend time actually sleeping.
         self.addCleanup(utilities.sleep.disable_mock)
         utilities.sleep.enable_mock()
 
     def get_touch_with_mocked_backend(self):
         touch = _uinput.Touch(device_class=Mock)
-        touch._device.mock_add_spec(
-            _uinput._UInputTouchDevice, spec_set=True)
+        touch._device.mock_add_spec(_uinput._UInputTouchDevice)
         return touch
 
-    def test_tap_must_put_finger_down_and_then_up(self):
+
+class UInputTouchFingerCoordinatesTestCase(
+        testscenarios.TestWithScenarios, UInputTouchBaseTestCase):
+
+    TEST_X_DESTINATION = 10
+    TEST_Y_DESTINATION = 10
+
+    scenarios = [
+        ('tap', {
+            'method': 'tap',
+            'args': (TEST_X_DESTINATION, TEST_Y_DESTINATION)
+        }),
+        ('press', {
+            'method': 'press',
+            'args': (TEST_X_DESTINATION, TEST_Y_DESTINATION)
+        }),
+        ('move', {
+            'method': 'move',
+            'args': (TEST_X_DESTINATION, TEST_Y_DESTINATION)
+        }),
+        ('drag', {
+            'method': 'drag',
+            'args': (0, 0, TEST_X_DESTINATION, TEST_Y_DESTINATION)
+        })
+    ]
+
+    def call_scenario_method(self, object_, method, *args):
+        getattr(object_, method)(*self.args)
+
+    def test_method_must_update_finger_coordinates(self):
+        touch = self.get_touch_with_mocked_backend()
+
+        self.call_scenario_method(touch, self.method, *self.args)
+
+        self.assertEqual(touch.x, self.TEST_X_DESTINATION)
+        self.assertEqual(touch.y, self.TEST_Y_DESTINATION)
+
+
+class UInputTouchTestCase(UInputTouchBaseTestCase):
+    """Test UInput Touch helper for autopilot tests."""
+
+    def test_initial_coordinates_must_be_zero(self):
+        touch = self.get_touch_with_mocked_backend()
+
+        self.assertEqual(touch.x, 0)
+        self.assertEqual(touch.y, 0)
+
+    def test_tap_must_put_finger_down_then_sleep_and_then_put_finger_up(self):
         expected_calls = [
             call.finger_down(0, 0),
+            call.sleep(ANY),
             call.finger_up()
         ]
 
         touch = self.get_touch_with_mocked_backend()
-        touch.tap(0, 0)
+        with patch('autopilot.input._uinput.sleep') as mock_sleep:
+            touch._device.attach_mock(mock_sleep, 'sleep')
+            touch.tap(0, 0)
         self.assertEqual(expected_calls, touch._device.mock_calls)
 
     def test_tap_object_must_put_finger_down_and_then_up_on_the_center(self):
-        object_ = type('Dummy', (object,), {'globalRect': (0, 0, 10, 10)})
+        object_ = make_fake_object(center=True)
         expected_calls = [
-            call.finger_down(5, 5),
+            call.finger_down(object_.center_x, object_.center_y),
             call.finger_up()
         ]
 
@@ -698,6 +766,38 @@ class UInputTouchTestCase(TestCase):
         touch.move(10, 10)
         self.assertEqual(expected_calls, touch._device.mock_calls)
 
+    def test_move_must_move_with_specified_rate(self):
+        expected_calls = [
+            call.finger_move(5, 5),
+            call.finger_move(10, 10),
+            call.finger_move(15, 15),
+        ]
+
+        touch = self.get_touch_with_mocked_backend()
+        touch.move(15, 15, rate=5)
+
+        self.assertEqual(
+            expected_calls, touch._device.mock_calls)
+
+    def test_move_without_rate_must_use_default(self):
+        expected_calls = [
+            call.finger_move(10, 10),
+            call.finger_move(20, 20),
+        ]
+
+        touch = self.get_touch_with_mocked_backend()
+        touch.move(20, 20)
+
+        self.assertEqual(
+            expected_calls, touch._device.mock_calls)
+
+    def test_move_to_same_place_must_not_move(self):
+        expected_calls = []
+
+        touch = self.get_touch_with_mocked_backend()
+        touch.move(0, 0)
+        self.assertEqual(expected_calls, touch._device.mock_calls)
+
     def test_drag_must_call_finger_down_move_and_up(self):
         expected_calls = [
             call.finger_down(0, 0),
@@ -709,42 +809,50 @@ class UInputTouchTestCase(TestCase):
         touch.drag(0, 0, 10, 10)
         self.assertEqual(expected_calls, touch._device.mock_calls)
 
-    def test_drag_must_move_with_specified_rate(self):
-        expected_calls = [
-            call.finger_down(0, 0),
-            call.finger_move(5, 5),
-            call.finger_move(10, 10),
-            call.finger_move(15, 15),
-            call.finger_up()]
-
+    def test_tap_without_press_duration_must_sleep_default_time(self):
         touch = self.get_touch_with_mocked_backend()
-        touch.drag(0, 0, 15, 15, rate=5)
 
-        self.assertEqual(
-            expected_calls, touch._device.mock_calls)
+        touch.tap(0, 0)
 
-    def test_drag_without_rate_must_use_default(self):
-        expected_calls = [
-            call.finger_down(0, 0),
-            call.finger_move(10, 10),
-            call.finger_move(20, 20),
-            call.finger_up()]
+        self.assertEqual(utilities.sleep.total_time_slept(), 0.1)
 
+    def test_tap_with_press_duration_must_sleep_specified_time(self):
         touch = self.get_touch_with_mocked_backend()
-        touch.drag(0, 0, 20, 20)
 
-        self.assertEqual(
-            expected_calls, touch._device.mock_calls)
+        touch.tap(0, 0, press_duration=10)
 
-    def test_drag_to_same_place_must_not_move(self):
-        expected_calls = [
-            call.finger_down(0, 0),
-            call.finger_up()
-        ]
+        self.assertEqual(utilities.sleep.total_time_slept(), 10)
 
+    def test_tap_object_without_duration_must_call_tap_with_default_time(self):
+        object_ = make_fake_object(center=True)
         touch = self.get_touch_with_mocked_backend()
-        touch.drag(0, 0, 0, 0)
-        self.assertEqual(expected_calls, touch._device.mock_calls)
+
+        with patch.object(touch, 'tap') as mock_tap:
+            touch.tap_object(object_)
+
+        mock_tap.assert_called_once_with(
+            object_.center_x,
+            object_.center_y,
+            press_duration=0.1,
+            time_between_events=0.1
+        )
+
+    def test_tap_object_with_duration_must_call_tap_with_specified_time(self):
+        object_ = make_fake_object(center=True)
+        touch = self.get_touch_with_mocked_backend()
+
+        with patch.object(touch, 'tap') as mock_tap:
+            touch.tap_object(
+                object_,
+                press_duration=10
+            )
+
+        mock_tap.assert_called_once_with(
+            object_.center_x,
+            object_.center_y,
+            press_duration=10,
+            time_between_events=0.1
+        )
 
 
 class MultipleUInputTouchBackend(_uinput._UInputTouchDevice):
@@ -781,55 +889,56 @@ class MultipleUInputTouchTestCase(TestCase):
         self.assertFalse(finger2.pressed)
 
 
-class DragUInputTouchTestCase(testscenarios.TestWithScenarios, TestCase):
+class MoveWithAnimationUInputTouchTestCase(
+        testscenarios.TestWithScenarios, TestCase):
 
     scenarios = [
-        ('drag to top', dict(
+        ('move to top', dict(
             start_x=50, start_y=50, stop_x=50, stop_y=30,
             expected_moves=[call.finger_move(50, 40),
                             call.finger_move(50, 30)])),
-        ('drag to bottom', dict(
+        ('move to bottom', dict(
             start_x=50, start_y=50, stop_x=50, stop_y=70,
             expected_moves=[call.finger_move(50, 60),
                             call.finger_move(50, 70)])),
-        ('drag to left', dict(
+        ('move to left', dict(
             start_x=50, start_y=50, stop_x=30, stop_y=50,
             expected_moves=[call.finger_move(40, 50),
                             call.finger_move(30, 50)])),
-        ('drag to right', dict(
+        ('move to right', dict(
             start_x=50, start_y=50, stop_x=70, stop_y=50,
             expected_moves=[call.finger_move(60, 50),
                             call.finger_move(70, 50)])),
 
-        ('drag to top-left', dict(
+        ('move to top-left', dict(
             start_x=50, start_y=50, stop_x=30, stop_y=30,
             expected_moves=[call.finger_move(40, 40),
                             call.finger_move(30, 30)])),
-        ('drag to top-right', dict(
+        ('move to top-right', dict(
             start_x=50, start_y=50, stop_x=70, stop_y=30,
             expected_moves=[call.finger_move(60, 40),
                             call.finger_move(70, 30)])),
-        ('drag to bottom-left', dict(
+        ('move to bottom-left', dict(
             start_x=50, start_y=50, stop_x=30, stop_y=70,
             expected_moves=[call.finger_move(40, 60),
                             call.finger_move(30, 70)])),
-        ('drag to bottom-right', dict(
+        ('move to bottom-right', dict(
             start_x=50, start_y=50, stop_x=70, stop_y=70,
             expected_moves=[call.finger_move(60, 60),
                             call.finger_move(70, 70)])),
 
-        ('drag less than rate', dict(
+        ('move less than rate', dict(
             start_x=50, start_y=50, stop_x=55, stop_y=55,
             expected_moves=[call.finger_move(55, 55)])),
 
-        ('drag with last move less than rate', dict(
+        ('move with last move less than rate', dict(
             start_x=50, start_y=50, stop_x=65, stop_y=65,
             expected_moves=[call.finger_move(60, 60),
                             call.finger_move(65, 65)])),
     ]
 
     def setUp(self):
-        super(DragUInputTouchTestCase, self).setUp()
+        super(MoveWithAnimationUInputTouchTestCase, self).setUp()
         # Mock the sleeps so we don't have to spend time actually sleeping.
         self.addCleanup(utilities.sleep.disable_mock)
         utilities.sleep.enable_mock()
@@ -843,12 +952,12 @@ class DragUInputTouchTestCase(testscenarios.TestWithScenarios, TestCase):
     def test_drag_moves(self):
         touch = self.get_touch_with_mocked_backend()
 
-        touch.drag(
-            self.start_x, self.start_y, self.stop_x, self.stop_y)
+        touch.press(self.start_x, self.start_y)
+        touch.move(self.stop_x, self.stop_y)
 
-        # We don't check the finger down and finger up. They are already
-        # tested.
-        expected_calls = [ANY] + self.expected_moves + [ANY]
+        expected_calls = (
+            [call.finger_down(self.start_x, self.start_y)] +
+            self.expected_moves)
         self.assertEqual(
             expected_calls, touch._device.mock_calls)
 
@@ -862,6 +971,30 @@ class PointerWithTouchBackendTestCase(TestCase):
         pointer = autopilot.input.Pointer(touch)
         return pointer
 
+    def test_initial_coordinates_must_be_zero(self):
+        pointer = self.get_pointer_with_touch_backend_with_mock_device()
+
+        self.assertEqual(pointer.x, 0)
+        self.assertEqual(pointer.y, 0)
+
+    def test_drag_must_call_move_with_animation(self):
+        test_rate = 2
+        test_time_between_events = 1
+        test_destination_x = 20
+        test_destination_y = 20
+
+        pointer = self.get_pointer_with_touch_backend_with_mock_device()
+        with patch.object(pointer._device, 'move') as mock_move:
+            pointer.drag(
+                0, 0,
+                test_destination_x, test_destination_y,
+                rate=test_rate, time_between_events=test_time_between_events)
+
+        mock_move.assert_called_once_with(
+            test_destination_x, test_destination_y,
+            animate=True,
+            rate=test_rate, time_between_events=test_time_between_events)
+
     def test_drag_with_rate(self):
         pointer = self.get_pointer_with_touch_backend_with_mock_device()
         with patch.object(pointer._device, 'drag') as mock_drag:
@@ -885,3 +1018,64 @@ class PointerWithTouchBackendTestCase(TestCase):
 
         mock_drag.assert_called_once_with(
             0, 0, 20, 20, rate=10, time_between_events=0.01)
+
+    def test_click_with_default_press_duration(self):
+        pointer = self.get_pointer_with_touch_backend_with_mock_device()
+        with patch.object(pointer._device, 'tap') as mock_tap:
+            pointer.click(1)
+
+        mock_tap.assert_called_once_with(
+            0, 0, press_duration=0.1, time_between_events=0.1)
+
+    def test_press_with_specified_press_duration(self):
+        pointer = self.get_pointer_with_touch_backend_with_mock_device()
+        with patch.object(pointer._device, 'tap') as mock_tap:
+            pointer.click(1, press_duration=10)
+
+        mock_tap.assert_called_once_with(
+            0, 0, press_duration=10, time_between_events=0.1)
+
+    def test_not_pressed_move_must_not_move_pointing_figer(self):
+        """Test for moving the finger when it is not pressed.
+
+        The move method on the pointer class must update the finger coordinates
+        but it must not execute a move on the device.
+
+        """
+        test_x_destination = 20
+        test_y_destination = 20
+        pointer = self.get_pointer_with_touch_backend_with_mock_device()
+
+        pointer.move(10, 10)
+        pointer._device._device.pressed = False
+
+        with patch.object(pointer._device._device, 'finger_move') as mock_move:
+            pointer.move(test_x_destination, test_y_destination)
+
+        self.assertFalse(mock_move.called)
+        self.assertEqual(pointer.x, test_x_destination)
+        self.assertEqual(pointer.y, test_y_destination)
+
+    def test_pressed_move_must_move_pointing_finger(self):
+        test_x_destination = 20
+        test_y_destination = 20
+
+        pointer = self.get_pointer_with_touch_backend_with_mock_device()
+
+        pointer.move(10, 10)
+        pointer._device._device.pressed = True
+
+        with patch.object(pointer._device._device, 'finger_move') as mock_move:
+            pointer.move(test_x_destination, test_y_destination)
+
+        mock_move.assert_called_once_with(20, 20)
+        self.assertEqual(pointer.x, test_x_destination)
+        self.assertEqual(pointer.y, test_y_destination)
+
+    def test_press_must_put_finger_down_at_last_move_position(self):
+        pointer = self.get_pointer_with_touch_backend_with_mock_device()
+        pointer.move(10, 10)
+
+        pointer.press()
+
+        pointer._device._device.finger_down.assert_called_once_with(10, 10)
